#------------------------------------------------------------------------------
# Programm         : T k T o o l s . p m
#------------------------------------------------------------------------------
# Autor            : Christian Dhl
# Erstellt         : 12. Februar 2004
# Zuletzt gendert : 18. Januar 2005
# Aufgabe          : Tk Hilfsfunktionen.
#------------------------------------------------------------------------------

package Graph::GED::TkTools;

use strict;
use warnings;


#------------------------------------------------------------------------------
# Von Exporter erben:
#------------------------------------------------------------------------------
use base qw/Exporter/;
our @EXPORT_OK = qw/anzeige_fenster farbwahl/;


#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug  = 0;


sub anzeige_fenster ($$$$$;$$$) {
    #--------------------------------------------------------------------------
    # sub          : a n z e i g e _ f e n s t e r
    #--------------------------------------------------------------------------
    # Autor        : CD
    # Aufgabe      : Zeigt etwas in einem Extra-Fenster an.
    # Parameter    : Notwendig:
    #                  1) Main-Widget
    #                  2) Titel
    #                  3) Label
    #                  4) Font
    #                  5) anzuzeigender Text
    #                Optional:
    #                  6) Modal            (default: 1 = ja   )
    #                  7) Breite           (default: 80       )
    #                  8) Hhe             (default: 35       )
    #                  9) Textfarbe        (default: 'blue'   )
    #                 10) Hintergrundfarbe (default: '#d7d7d7')
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 18.11.2003 - CD - Erstellt
    # 0.0.2 - 20.11.2003 - CD - Bindungen fr PageUp, PageDn, Home und End...
    #                           Optionale Parameter fr Breite, Hhe, Textfarbe
    #                           und Hintergrundfarbe eingebaut.
    # 0.0.3 - 04.12.2003 - CD - Verbesserungen: Button in Extraframe, das vor
    #                           dem Textfeld gepackt wird, beim Expanden wird
    #                           die Zeile mit dem Schalter nicht mehr ver-
    #                           grert.
    # 0.0.4 - 08.02.2004 - CD - In ged.pl kopiert und an %Konfig angepasst.
    # 0.0.5 - 12.02.2004 - CD - Nach Graph::GED::TkTools verschoben und die
    #                           Parameter Main-Widget und Modal eingebaut.
    # 0.0.6 - 03.01.2005 - CD - Weitere Bindungen fr schneres Navigieren
    #                           eingebaut, insbesondere frs Mausrad.
    #--------------------------------------------------------------------------

    my $mw     = shift;
    my $titel  = shift;
    my $label  = shift;
    my $font   = shift;
    my $text   = shift;
    my $modal  = shift;
       $modal  = 0 unless defined $modal; # damit 0 bergeben werden kann
    my $width  = shift || 80;
    my $height = shift || 35;
    my $tcolor = shift || 'blue';
    my $bcolor = shift || '#d7d7d7';

    print "Graph::GED::TkTools::anzeige_fenster(\n",
          "                Titel : $titel\n",
          "                Label : $label\n",
          "                Font  : $font\n",
          "               )\n"
          if $Debug;


    #--------------------------------------------------------------------------
    # Neues Toplevel-Fenster erzeugen, das den Text anzeigt:
    #--------------------------------------------------------------------------
    my $tl = $mw->Toplevel();
    $tl->title($titel);
    $tl->Label(-text   => $label,
               #-anchor => 'w',
              )
        ->pack(-anchor => 'w');
    my $fr = $tl->Frame()
                 ->pack(-expand => '0',
                        -fill   => 'x',
                        -side   => 'bottom',
                       );
    my $txt = $tl->Scrolled("Text",
                            -scrollbars => 'osoe',
                            -background => $bcolor,
                            -foreground => $tcolor,
                            -state      => 'disabled',
                            -width      => $width,
                            -height     => $height,
                            -relief     => 'flat',
                            -font       => $font,
                            -wrap       => 'none',
                           )
                     ->pack(-side       => 'top',
                            -expand     => 1,
                            -fill       => 'both',
                           );
    my $ok = $fr->Button(-text        => 'OK',
                         -command     => sub {$tl->destroy();
                                              #$Konfig{mw}
                                              #  ->bind('<Escape>', \&beenden);
                                              $mw->focus();
                                             },
                         -default     => 'active',
                         -padx        => 15,
                        )
                  ->pack(-expand      => '0',
                         -fill        => 'none',
                         -side        => 'right',
                        );

    $txt->configure(-state => 'normal');
    $txt->delete('1.0', 'end');
    $txt->insert('end', $text);
    #$txt->insert('end', "\n");
    $txt->configure(-state => 'disabled');

    #
    # Bindungen zum $MW fr Escape lsen und an Lschen dieses Fensters binden.
    # (Beim Beenden des Fensters wird ESC wieder an $MW gebunden (siehe oben).)
    #
    $tl->bind('<Return>', sub{$ok->invoke()});
    #$mw->bind ('<Escape>', '');
    $tl->bind('<Escape>', sub{$ok->invoke()});
    #        *$text*->yviewScroll(*number, what*)
    #            This command adjust the view in the window up or down
    #            according to *number* and *what*. *Number* must be an
    #            integer. *What* must be either units or pages. If *what* is
    #            units, the view adjusts up or down by *number* lines on the
    #            display; if it is pages then the view adjusts by *number*
    #            screenfuls. If *number* is negative then earlier positions
    #            in the text become visible; if it is positive then later
    #            positions in the text become visible.
    #$tl->bind('<Prior>', sub { $txt->yviewScroll(-1, 'units') } );
    #$tl->bind('<Next>',  sub { $txt->yviewScroll( 1, 'units') } );
    ##$tl->bind('<Home>',  sub { $txt->see('1.0') } );
    ##$tl->bind('<End>',   sub { $txt->see('end') } );
    $tl->bind('<Home>',  sub { $txt->yviewMoveto(0) } );
    $tl->bind('<End>',   sub { $txt->yviewMoveto(1) } );
    # geht beides, unteres wohl etwas schneller
    #$tl->bind('<Left>',  sub { $txt->yviewScroll(-1, 'units') } );
    #$tl->bind('<Up>',    sub { $txt->yviewScroll(-1, 'units') } );
    #$tl->bind('<Right>', sub { $txt->yviewScroll( 1, 'units') } );
    #$tl->bind('<Down>',  sub { $txt->yviewScroll( 1, 'units') } );
    ## irgendwas stimmt noch nicht ganz... da mssen eingebaute Funktionalitten
    ## mit reinspielen...

    $tl->bind('<Left>',          sub { $txt->yviewScroll(-1, 'units') } );
    $tl->bind('<Up>',            sub { $txt->yviewScroll(-1, 'units') } );
    $tl->bind('<Down>',          sub { $txt->yviewScroll( 1, 'units') } );
    $tl->bind('<Right>',         sub { $txt->yviewScroll( 1, 'units') } );
    $tl->bind('<Alt-Up>',        sub { $txt->yviewScroll(-3, 'units') } );
    $tl->bind('<Alt-Down>',      sub { $txt->yviewScroll( 3, 'units') } );
    $tl->bind('<Prior>',         sub { $txt->yviewScroll(-1, 'pages') } );
    $tl->bind('<Next>',          sub { $txt->yviewScroll( 1, 'pages') } );
    $tl->bind('<MouseWheel>',    [ sub { $txt->yviewScroll(-($_[1]/120)*3, 'units') }, Tk::Ev('D') ] );

    #$tl->bind('<F9>',    sub { $txt->yviewScroll(-1, 'units') } );
    #$tl->bind('<F10>',   sub { $txt->yviewScroll( 1, 'units') } );
    #$tl->bind('<F5>',    sub { $txt->yviewScroll(-1, 'pages') } );
    #$tl->bind('<F6>',    sub { $txt->yviewScroll( 1, 'pages') } );

    #
    # Rest:
    # (grab verhindert das zurckschalten auf $WM per Klick, focus sorgt dafr,
    # dass $tl auch wirklich aktiv ist.)
    #
    $tl->grab() if $modal;
    $tl->focus();
    $txt->focus(); # ?
} # sub Graph::GED::TkTools::anzeige_fenster


sub farbwahl ($$$$) {
    #--------------------------------------------------------------------------
    # sub          : f a r b w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dhl
    # Aufgabe      : Kleine Farbauswahl.
    # Parameter    : 1) Main-Widget
    #                2) absolute x-Koordinate fr das Fenster
    #                3) absolute y-Koordinate fr das Fenster
    #                4) Funktionsreferenz, die FUnktion wird aufgerufen, wenn
    #                   eine Farbe ausgewhlt wird.
    # Rckgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 02.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($mw, $ax, $ay, $sub) = @_;

    print "Graph::GED::TkTools::farbwahl ($ax, $ay) mw=$mw sub=$sub\n"
        if $Debug;


    my @farben = map { "#$_" } (qw/000000 AA0000 00AA00 0000AA
                                   444444 FF0000 00FF00 0000FF
                                   888888 FFFF00 00FFFF FF00FF
                                   CCCCCC FFAA00 00BBBB 880088
                                   FFFFFF AA8866 008888 660066/);
    my $f = $farben[0];

    my $menu = $mw->Menu(-tearoff => 0);

    for (@farben) {
        $menu->radiobutton(
            -label     => $_,
            -variable  => \$f,
           #-command   => [ \&$sub, $f ],
            -command   => sub { $sub->($f) },
        );
    }

    my $topborder    = 'gray50';
    my $bottomborder = 'gray75';

    for (0 .. $#farben) {
        #----------------------------------------------------------------------
        # Create a 16 pixel x 16 pixel solid color swatch.
        # Add a black ring around the currently selected item.
        #----------------------------------------------------------------------

        my $color = $menu->entrycget($_, -label);
        my $p = $mw->Photo(qw/-width 23 -height 16/);
        $p->put($topborder,    qw/-to  0  0 16  1/);
        $p->put($topborder,    qw/-to  0  1  1 16/);
        $p->put($bottomborder, qw/-to  1 15 16 16/);
        $p->put($bottomborder, qw/-to 15  1 16 15/);
        $p->put($color,        qw/-to  1  1 15 15/);

        my $r = $mw->Photo(qw/-width 16 -height 16/);
        $r->put(qw/black          -to  0  0 16  2/);
        $r->put(qw/black          -to  0  2  2 16/);
        $r->put(qw/black          -to  2 14 16 16/);
        $r->put(qw/black          -to 14  2 16 14/);
        $r->put($color       , qw/-to  2  2 14 14/);

        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 4;
        $menu->entryconfigure($_,
            -image       => $p,
            -hidemargin  => 1,
            -selectimage => $r,
        );
    }

    $menu->post($ax, $ay);
} # sub Graph::GED::TkTools::farbwahl


1;

__END__

=head1 NAME

Graph::GED::TkTools.pm - a module with a few tools for Tk

=head1 FUNCTIONS

=over

=item C<anzeige_fenster>

    A basic function for windows that show something.

=item C<farbwahl>

    A color chooser.

=back

=head1 COPYRIGHT

Copyright 2004-2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
